// Copyright 2000-2020 JetBrains s.r.o. Use of this source code is governed by the Apache 2.0 license that can be found in the LICENSE file.

package com.intellij.xml;

import com.intellij.psi.PsiElement;
import com.intellij.psi.meta.PsiMetaData;
import com.intellij.psi.xml.XmlElement;
import com.intellij.util.ArrayFactory;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.Collection;
import java.util.Collections;

public interface XmlAttributeDescriptor extends PsiMetaData {
  XmlAttributeDescriptor[] EMPTY = new XmlAttributeDescriptor[0];
  ArrayFactory<XmlAttributeDescriptor> ARRAY_FACTORY = count -> count == 0 ? EMPTY : new XmlAttributeDescriptor[count];

  boolean isRequired();
  boolean isFixed();
  boolean hasIdType();
  boolean hasIdRefType();

  @Nullable
  String getDefaultValue();

  //todo: refactor to hierarchy of value descriptor?
  boolean isEnumerated();
  String @Nullable [] getEnumeratedValues();

  @Nullable
  String validateValue(XmlElement context, String value);

  @NotNull
  default Collection<PsiElement> getDeclarations() {
    PsiElement declaration = getDeclaration();
    return declaration != null ? Collections.singleton(declaration)
                               : Collections.emptyList();
  }
}
